@@ -1,6 +1,6 @@
 /*      Orage - Calendar and alarm handler
  *
- * Copyright (c) 2005-2008 Juha Kautto  (juha at xfce.org)
+ * Copyright (c) 2005-2011 Juha Kautto  (juha at xfce.org)
  * Copyright (c) 2003-2005 Mickael Graf (korbinus at xfce.org)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -33,20 +33,24 @@
 
 #include <glib.h>
 #include <glib/gprintf.h>
+#include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
-
+/*
 #include <libxfce4util/libxfce4util.h>
 #include <libxfcegui4/libxfcegui4.h>
+*/
 
 #include "orage-i18n.h"
 #include "functions.h"
 #include "parameters.h"
 
-/**************************************
- *  General purpose helper functions  *
- **************************************/
+/* message logging level */
+int g_log_level=0;
 
+/**************************************
+ *  Debugging helping functions       *
+ **************************************/
 /* this is for testing only. it can be used to see where time is spent.
  * Add call program_log("dbus started") in the code and run orage like:
  * strace -ttt -f -o /tmp/logfile.strace ./orage
@@ -54,6 +58,7 @@
  * grep MARK /tmp/logfile.strace
  * grep MARK /tmp/logfile.strace|sed s/", F_OK) = -1 ENOENT (No such file or directory)"/\)/
  * */
+/*
 void program_log (const char *format, ...)
 {
         va_list args;
@@ -69,6 +74,7 @@ void program_log (const char *format, ...)
         access (str, F_OK);
         g_free (str);
 }
+*/
 
 /* Print message at various level:
  * < 0     = Debug (Use only in special debug code which should not be in 
@@ -77,14 +83,14 @@ void program_log (const char *format, ...)
  * 100-199 = Warning
  * 200-299 = Critical warning
  * 300-    = Error
- * variable g_par.log_level can be used to control how much data is printed
+ * variable g_log_level can be used to control how much data is printed
  */
 void orage_message(gint level, const char *format, ...)
 {
     va_list args;
     char *formatted;
 
-    if (level < g_par.log_level)
+    if (level < g_log_level)
         return;
     va_start(args, format);
     formatted = g_strdup_vprintf(format, args);
@@ -103,6 +109,10 @@ void orage_message(gint level, const char *format, ...)
     g_free(formatted);
 }
 
+/**************************************
+ *  General purpose helper functions  *
+ **************************************/
+
 GtkWidget *orage_create_combo_box_with_content(char *text[], int size)
 {
     register int i;
@@ -427,6 +437,38 @@ GtkWidget *orage_period_hbox_new(gboolean head_space, gboolean tail_space
     return hbox;
 }
 
+GtkWidget *orage_create_framebox_with_content(const gchar *title
+        , GtkWidget *content)
+{
+    GtkWidget *framebox;
+    GtkWidget *frame_bin;
+    gchar *tmp;
+    GtkWidget *label;
+
+    framebox = gtk_frame_new(NULL);
+    gtk_frame_set_shadow_type(GTK_FRAME(framebox), GTK_SHADOW_NONE);
+    gtk_frame_set_label_align(GTK_FRAME(framebox), 0.0, 1.0);
+    gtk_widget_show(framebox);
+
+    if (title) {
+        tmp = g_strdup_printf("<b>%s</b>", title);
+        label = gtk_label_new(tmp);
+        gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+        gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+        gtk_widget_show(label);
+        gtk_frame_set_label_widget(GTK_FRAME(framebox), label);
+        g_free(tmp);
+    }
+
+    frame_bin = gtk_alignment_new(0.0, 0.0, 1.0, 1.0);
+    gtk_alignment_set_padding(GTK_ALIGNMENT(frame_bin), 5, 5, 21, 5);
+    gtk_widget_show(frame_bin);
+    gtk_container_add(GTK_CONTAINER(framebox), frame_bin);
+    gtk_container_add(GTK_CONTAINER(frame_bin), content);
+
+    return(framebox);
+}
+
 /*******************************************************
  * time convert and manipulation functions
  *******************************************************/
@@ -615,7 +657,7 @@ char *orage_i18_date_to_icaldate(const char *i18_date)
     return(icalt);
 }
 
-struct tm *orage_localtime()
+struct tm *orage_localtime(void)
 {
     time_t tt;
 
@@ -623,7 +665,7 @@ struct tm *orage_localtime()
     return(localtime(&tt));
 }
 
-char *orage_localdate_i18()
+char *orage_localdate_i18(void)
 {
     struct tm *t;
 
@@ -634,7 +676,7 @@ char *orage_localdate_i18()
 /* move one day forward or backward */
 void orage_move_day(struct tm *t, int day)
 {
-    guint monthdays[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
+    gint monthdays[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
 
     if (day < -1 || day > 1) {
         g_warning("orage: orage_move_day wrong parameter %d", day);
@@ -691,7 +733,7 @@ void orage_select_date(GtkCalendar *cal
     if (cur_year == year && cur_month == month)
         gtk_calendar_select_day(cal, day);
     else {
-        gtk_calendar_select_day(cal, 0); /* needed to avoid illegal day/month */
+        gtk_calendar_select_day(cal, 0); /* need to avoid illegal day/month */
         gtk_calendar_select_month(cal, month, year);
         gtk_calendar_select_day(cal, day);
     }
@@ -706,164 +748,354 @@ void orage_select_today(GtkCalendar *cal)
 }
 
 /*******************************************************
- * rc file interface
+ * data and config file locations
  *******************************************************/
 
+gboolean orage_copy_file(gchar *source, gchar *target)
+{
+    gchar *text;
+    gsize text_len;
+    GError *error = NULL;
+    gboolean ok = TRUE;
+
+    /* read file */
+    if (!g_file_get_contents(source, &text, &text_len, &error)) {
+        orage_message(150, "orage_copy_file: Could not open file (%s) error:%s"
+                , source, error->message);
+        g_error_free(error);
+        ok = FALSE;
+    }
+    /* write file */
+    if (!g_file_set_contents(target, text, -1, &error)) {
+        orage_message(150, "orage_copy_file: Could not write file (%s) error:%s"
+                , target, error->message);
+        g_error_free(error);
+        ok = FALSE;
+    }
+    g_free(text);
+    return(ok);
+}
+
+gchar *orage_xdg_system_data_file_location(char *name)
+{
+    char *file_name;
+    const gchar * const *base_dirs;
+    int i;
+
+    base_dirs = g_get_system_data_dirs();
+    for (i = 0; base_dirs[i] != NULL; i++) {
+        file_name = g_build_filename(base_dirs[i], name, NULL);
+        if (g_file_test(file_name, G_FILE_TEST_IS_REGULAR)) {
+            return(file_name);
+        }
+        g_free(file_name);
+    }
+
+    /* no system wide data file */
+    return(NULL);
+}
+
+/* Returns valid xdg local data file name. 
+   If the file did not exist, it checks if there are system defaults 
+   and creates it from those */
 gchar *orage_data_file_location(char *name)
 {
-    char *file_name, *dir_name;
+    char *file_name, *dir_name, *sys_name;
+    const char *base_dir;
+    int mode = 0700;
 
-    dir_name = g_strconcat(ORAGE_DIR, name, NULL);
-    file_name = xfce_resource_save_location(XFCE_RESOURCE_DATA, dir_name
-            , TRUE);
-    g_free(dir_name);
-    return(file_name);
-}
-
-gchar *orage_config_file_location(char *name)
-{
-    char *file_name, *dir_name;
-
-    dir_name = g_strconcat(ORAGE_DIR, name, NULL);
-    file_name = xfce_resource_save_location(XFCE_RESOURCE_CONFIG, dir_name
-            , TRUE);
-    g_free(dir_name);
-    return(file_name);
-}
-
-OrageRc *orage_rc_file_open(char *fpath, gboolean read_only)
-{
-    XfceRc *rc;
-    OrageRc *orc = NULL;
-
-    if ((rc = xfce_rc_simple_open(fpath, read_only)) == NULL && read_only) {
-        orage_message(-90, "orage_rc_open: Unable to open (read) RC file (%s). Creating it.", fpath);
-        /* let's try to build it */
-        if ((rc = xfce_rc_simple_open(fpath, FALSE)) == NULL) {
-            /* still failed, can't do more */
-            orage_message(150, "orage_rc_open: Unable to open (write) RC file (%s).", fpath);
+    base_dir = g_get_user_data_dir();
+    file_name = g_build_filename(base_dir, name, NULL);
+    if (!g_file_test(file_name, G_FILE_TEST_IS_REGULAR)) {
+        /* it does not exist, let's try to create it */
+        dir_name = g_path_get_dirname((const gchar *)file_name);
+        if (g_mkdir_with_parents(dir_name, mode)) {
+            orage_message(150, "orage_data_file_location: (%s) (%s) directory creation failed.\n", base_dir, file_name);
+        }
+        g_free(dir_name);
+        /* now we have the directories ready, let's check for system default */
+        sys_name = orage_xdg_system_data_file_location(name);
+        if (sys_name) { /* we found it, we need to copy it */
+            orage_copy_file(sys_name, file_name);
         }
     }
 
-    if (rc) { /* we managed to open it */
-        orc = g_new(OrageRc, 1);
-        orc->rc = rc;
+    return(file_name);
+}
+
+/* these are default files and must only be read */
+gchar *orage_xdg_system_config_file_location(char *name)
+{
+    char *file_name;
+    const gchar * const *base_dirs;
+    int i;
+
+    base_dirs = g_get_system_config_dirs();
+    for (i = 0; base_dirs[i] != NULL; i++) {
+        file_name = g_build_filename(base_dirs[i], name, NULL);
+        if (g_file_test(file_name, G_FILE_TEST_IS_REGULAR)) {
+            return(file_name);
+        }
+        g_free(file_name);
     }
+
+    /* no system wide config file */
+    return(NULL);
+}
+
+/* Returns valid xdg local config file name. 
+   If the file did not exist, it checks if there are system defaults 
+   and creates it from those */
+gchar *orage_config_file_location(char *name)
+{
+    char *file_name, *dir_name, *sys_name;
+    const char *base_dir;
+    int mode = 0700;
+
+    base_dir = g_get_user_config_dir();
+    file_name = g_build_filename(base_dir, name, NULL);
+    if (!g_file_test(file_name, G_FILE_TEST_IS_REGULAR)) {
+        /* it does not exist, let's try to create it */
+        dir_name = g_path_get_dirname((const gchar *)file_name);
+        if (g_mkdir_with_parents(dir_name, mode)) {
+            orage_message(150, "orage_config_file_location: (%s) (%s) directory creation failed.\n", base_dir, file_name);
+        }
+        g_free(dir_name);
+        /* now we have the directories ready, let's check for system default */
+        sys_name = orage_xdg_system_config_file_location(name);
+        if (sys_name) { /* we found it, we need to copy it */
+            orage_copy_file(sys_name, file_name);
+        }
+    }
+
+    return(file_name);
+}
+
+/*******************************************************
+ * rc file interface
+ *******************************************************/
+
+OrageRc *orage_rc_file_open(char *fpath, gboolean read_only)
+{
+    /* XfceRc *rc; */
+    OrageRc *orc = NULL;
+    GKeyFile *grc;
+    GError *error = NULL;
+
+    grc = g_key_file_new();
+    if (g_key_file_load_from_file(grc, (const gchar *)fpath
+            , G_KEY_FILE_KEEP_COMMENTS, &error)) { /* success */
+        orc = g_new(OrageRc, 1);
+        orc->rc = grc;
+        orc->read_only = read_only;
+        orc->file_name = g_strdup(fpath);
+        orc->cur_group = NULL;
+    }
+    else {
+        orage_message(-90, "orage_rc_file_open: Unable to open RC file (%s). Creating it. (%s)", fpath, error->message);
+        g_clear_error(&error);
+        if (g_file_set_contents((const gchar *)fpath, "#Created by Orage", -1
+                    , &error)) { /* successfully created new file */
+            orc = g_new(OrageRc, 1);
+            orc->rc = grc;
+            orc->read_only = read_only;
+            orc->file_name = g_strdup(fpath);
+            orc->cur_group = NULL;
+        }
+        else {
+            orage_message(150, "orage_rc_file_open: Unable to open (create) RC file (%s). (%s)", fpath, error->message);
+            g_key_file_free(grc);
+        }
+    }
+
     return(orc);
 }
 
 void orage_rc_file_close(OrageRc *orc)
+    /* FIXME: check if file contents have been changed and only write when
+       needed or build separate save function */
 {
+    GError *error = NULL;
+    gchar *file_content = NULL;
+    gsize length;
+
     if (orc) {
-        xfce_rc_close((XfceRc *)orc->rc);
+        /* xfce_rc_close((XfceRc *)orc->rc); */
+        if (!orc->read_only) { /* need to write it */
+            file_content = g_key_file_to_data(orc->rc, &length, NULL);
+            if (file_content 
+            && !g_file_set_contents(orc->file_name, file_content, -1
+                , &error)) { /* write needed and failed */
+                orage_message(150, "orage_rc_file_close: File save failed. RC file (%s). (%s)", orc->file_name, error->message);
+            }
+            g_free(file_content);
+        }
+        g_key_file_free(orc->rc);
+        g_free((void *)orc->file_name);
+        g_free((void *)orc->cur_group);
         g_free(orc);
     }
+    else {
+        orage_message(-90, "orage_rc_file_close: closing empty file.");
+    }
 }
 
 gchar **orage_rc_get_groups(OrageRc *orc)
 {
-    return(xfce_rc_get_groups((XfceRc *)orc->rc));
+    return(g_key_file_get_groups((GKeyFile *)orc->rc, NULL));
 }
 
 void orage_rc_set_group(OrageRc *orc, char *grp)
 {
-    xfce_rc_set_group((XfceRc *)orc->rc, grp);
+    g_free((void *)orc->cur_group);
+    orc->cur_group = g_strdup(grp);
 }
 
 void orage_rc_del_group(OrageRc *orc, char *grp)
 {
-    xfce_rc_delete_group((XfceRc *)orc->rc, grp, FALSE);
+    GError *error = NULL;
+
+    if (!g_key_file_remove_group((GKeyFile *)orc->rc, (const gchar *)grp
+                , &error)) {
+        orage_message(150, "orage_rc_del_group: Group remove failed. RC file (%s). group (%s) (%s)", orc->file_name, grp, error->message);
+    }
 }
 
 gchar *orage_rc_get_group(OrageRc *orc)
 {
-    return(g_strdup(xfce_rc_get_group((XfceRc *)orc->rc)));
+    return(g_strdup(orc->cur_group));
 }
 
 gchar *orage_rc_get_str(OrageRc *orc, char *key, char *def)
 {
-    return(g_strdup(xfce_rc_read_entry((XfceRc *)orc->rc, key, def)));
+    GError *error = NULL;
+    gchar *ret;
+
+    ret = g_key_file_get_string((GKeyFile *)orc->rc, orc->cur_group
+            , (const gchar *)key, &error);
+    if (!ret && error) {
+        ret = g_strdup(def);
+        orage_message(-90, "orage_rc_get_str: str (%s) group (%s) in RC file (%s) not found, using default (%s)", key, orc->cur_group, orc->file_name, ret);
+    }
+    return(ret);
 }
 
 gint orage_rc_get_int(OrageRc *orc, char *key, gint def)
 {
-    return(xfce_rc_read_int_entry((XfceRc *)orc->rc, key, def));
+    GError *error = NULL;
+    gint ret;
+
+    ret = g_key_file_get_integer((GKeyFile *)orc->rc, orc->cur_group
+            , (const gchar *)key, &error);
+    if (!ret && error) {
+        ret = def;
+        orage_message(-90, "orage_rc_get_int: str (%s) group (%s) in RC file (%s) not found, using default (%d)", key, orc->cur_group, orc->file_name, ret);
+    }
+    return(ret);
 }
 
 gboolean orage_rc_get_bool(OrageRc *orc, char *key, gboolean def)
 {
-    return(xfce_rc_read_bool_entry((XfceRc *)orc->rc, key, def));
+    GError *error = NULL;
+    gboolean ret;
+
+    ret = g_key_file_get_boolean((GKeyFile *)orc->rc, orc->cur_group
+            , (const gchar *)key, &error);
+    if (!ret && error) {
+        ret = def;
+        orage_message(-90, "orage_rc_get_bool: str (%s) group (%s) in RC file (%s) not found, using default (%d)", key, orc->cur_group, orc->file_name, ret);
+    }
+    return(ret);
 }
 
 void orage_rc_put_str(OrageRc *orc, char *key, char *val)
 {
-    if (val != NULL)
-        xfce_rc_write_entry((XfceRc *)orc->rc, key, val);
+    if (ORAGE_STR_EXISTS(val))
+        g_key_file_set_string((GKeyFile *)orc->rc, orc->cur_group
+                , (const gchar *)key, (const gchar *)val);
 }
 
 void orage_rc_put_int(OrageRc *orc, char *key, gint val)
 {
-    xfce_rc_write_int_entry((XfceRc *)orc->rc, key, val);
+    g_key_file_set_integer((GKeyFile *)orc->rc, orc->cur_group
+            , (const gchar *)key, val);
 }
 
 void orage_rc_put_bool(OrageRc *orc, char *key, gboolean val)
 {
-    xfce_rc_write_bool_entry((XfceRc *)orc->rc, key, val);
+    g_key_file_set_boolean((GKeyFile *)orc->rc, orc->cur_group
+            , (const gchar *)key, val);
 }
 
 gboolean orage_rc_exists_item(OrageRc *orc, char *key)
 {
-    return(xfce_rc_has_entry((XfceRc *)orc->rc, key));
+    return(g_key_file_has_key((GKeyFile *)orc->rc, orc->cur_group
+            , (const gchar *)key, NULL));
 }
 
 void orage_rc_del_item(OrageRc *orc, char *key)
 {
-    xfce_rc_delete_entry((XfceRc *)orc->rc, key, FALSE);
+    g_key_file_remove_key((GKeyFile *)orc->rc, orc->cur_group
+            , (const gchar *)key, NULL);
 }
 
 /*******************************************************
- * xfce functions
+ * dialog functions
  *******************************************************/
 
 gint orage_info_dialog(GtkWindow *parent
         , char *primary_text, char *secondary_text)
 {
-    return(xfce_message_dialog(parent
-        , _("Info")
-        , GTK_STOCK_DIALOG_INFO
-        , primary_text
-        , secondary_text
-        , GTK_STOCK_OK, GTK_RESPONSE_ACCEPT
-        , NULL));
+    GtkWidget *dialog;
+    gint result;
+
+    dialog = gtk_message_dialog_new(parent
+            , GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT
+            , GTK_MESSAGE_INFO
+            , GTK_BUTTONS_OK
+            , "%s", primary_text);
+    if (secondary_text) 
+        gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog)
+                , "%s", secondary_text);
+    result = gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_widget_destroy(dialog);
+    return(result);
 }
 
 gint orage_warning_dialog(GtkWindow *parent
         , char *primary_text, char *secondary_text)
 {
-    return(xfce_message_dialog(parent
-        , _("Warning")
-        , GTK_STOCK_DIALOG_WARNING
-        , primary_text
-        , secondary_text
-        , GTK_STOCK_NO, GTK_RESPONSE_CANCEL
-        , GTK_STOCK_YES, GTK_RESPONSE_ACCEPT
-        , NULL));
+    GtkWidget *dialog;
+    gint result;
+
+    dialog = gtk_message_dialog_new(parent
+            , GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT
+            , GTK_MESSAGE_WARNING
+            , GTK_BUTTONS_YES_NO
+            , "%s", primary_text);
+    if (secondary_text) 
+        gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog)
+                , "%s", secondary_text);
+    result = gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_widget_destroy(dialog);
+    return(result);
 }
 
 gint orage_error_dialog(GtkWindow *parent
         , char *primary_text, char *secondary_text)
 {
-    return(xfce_message_dialog(parent
-        , _("Error")
-        , GTK_STOCK_DIALOG_ERROR
-        , primary_text
-        , secondary_text
-        , GTK_STOCK_OK, GTK_RESPONSE_ACCEPT
-        , NULL));
-}
+    GtkWidget *dialog;
+    gint result;
 
-GtkWidget *orage_create_framebox_with_content(const gchar *title
-        , GtkWidget *content)
-{
-    return(xfce_create_framebox_with_content (title, content));
+    dialog = gtk_message_dialog_new(parent
+            , GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT
+            , GTK_MESSAGE_ERROR
+            , GTK_BUTTONS_OK
+            , "%s", primary_text);
+    if (secondary_text) 
+        gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog)
+                , "%s", secondary_text);
+    result = gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_widget_destroy(dialog);
+    return(result);
 }
