@@ -1,6 +1,6 @@
 /*      Orage - Calendar and alarm handler
  *
- * Copyright (c) 2005-2008 Juha Kautto  (juha at xfce.org)
+ * Copyright (c) 2005-2011 Juha Kautto  (juha at xfce.org)
  * Copyright (c) 2004-2005 Mickael Graf (korbinus at xfce.org)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -56,7 +56,7 @@
 */
 
 
-gboolean orage_mark_appointments()
+gboolean orage_mark_appointments(void)
 {
 #undef P_N
 #define P_N "orage_mark_appointments: "
@@ -168,22 +168,39 @@ static void mView_selectToday_activate_cb(GtkMenuItem *menuitem
     orage_select_today(GTK_CALENDAR(cal->mCalendar));
 }
 
+static void mView_StartGlobaltime_activate_cb(GtkMenuItem *menuitem
+        , gpointer user_data)
+{
+#undef P_N
+#define P_N "mView_StartGlobaltime_activate_cb: "
+    GError *error = NULL;
+
+#ifdef ORAGE_DEBUG
+    orage_message(-100, P_N);
+#endif
+    if (!orage_exec("globaltime", FALSE, &error))
+        orage_message(100, "%s: start of %s failed: %s", "Orage", "globaltime"
+                , error->message);
+}
+
 static void mHelp_help_activate_cb(GtkMenuItem *menuitem, gpointer user_data)
 {
 #undef P_N
 #define P_N "mHelp_help_activate_cb: "
     gchar *helpdoc;
+    GError *error = NULL;
 
 #ifdef ORAGE_DEBUG
     orage_message(-100, P_N);
 #endif
-    helpdoc = g_strconcat("xfbrowser4 ", PACKAGE_DATA_DIR
+    helpdoc = g_strconcat("firefox ", PACKAGE_DATA_DIR
            , G_DIR_SEPARATOR_S, "orage"
            , G_DIR_SEPARATOR_S, "doc"
            , G_DIR_SEPARATOR_S, "C"
            , G_DIR_SEPARATOR_S, "orage.html"
            , NULL);
-    orage_exec(helpdoc, NULL, NULL);
+    if (!orage_exec(helpdoc, FALSE, &error))
+        orage_message(100, "start of %s failed: %s", helpdoc, error->message);
     g_free(helpdoc);
 }
 
@@ -300,6 +317,12 @@ static void build_menu(void)
             orage_menu_item_new_with_mnemonic(_("Select _Today")
                     , cal->mView_menu);
 
+    menu_separator = orage_separator_menu_item_new(cal->mView_menu);
+
+    cal->mView_StartGlobaltime = 
+            orage_menu_item_new_with_mnemonic(_("Show _Globaltime")
+                    , cal->mView_menu);
+
     /* Help menu */
     cal->mHelp_menu = orage_menu_new(_("_Help"), cal->mMenubar);
     cal->mHelp_help = orage_image_menu_item_new_from_stock("gtk-help"
@@ -326,6 +349,8 @@ static void build_menu(void)
             , G_CALLBACK(mView_ViewSelectedWeek_activate_cb),(gpointer) cal);
     g_signal_connect((gpointer) cal->mView_selectToday, "activate"
             , G_CALLBACK(mView_selectToday_activate_cb),(gpointer) cal);
+    g_signal_connect((gpointer) cal->mView_StartGlobaltime, "activate"
+            , G_CALLBACK(mView_StartGlobaltime_activate_cb),(gpointer) cal);
     g_signal_connect((gpointer) cal->mHelp_help, "activate"
             , G_CALLBACK(mHelp_help_activate_cb), NULL);
     g_signal_connect((gpointer) cal->mHelp_about, "activate"
@@ -473,8 +498,8 @@ static void info_process(gpointer a, gpointer pbox)
 {
 #undef P_N
 #define P_N "info_process: "
-    xfical_appt *appt = (xfical_appt *)a;;
-    GtkBox *box = GTK_BOX(pbox);
+    xfical_appt *appt = (xfical_appt *)a;
+    GtkBox *box= GTK_BOX(pbox);
     CalWin *cal = (CalWin *)g_par.xfcal;
     gboolean todo;
 
@@ -685,7 +710,7 @@ static void mCalendar_day_selected_cb(GtkCalendar *calendar
     build_mainbox_event_box();
 }
 
-void build_mainbox_event_box()
+void build_mainbox_event_box(void)
 {
 #undef P_N
 #define P_N "build_mainbox_event_box: "
@@ -716,7 +741,7 @@ void build_mainbox_info(void)
     build_mainbox_event_info();
 }
 
-void build_mainWin()
+void build_mainWin(void)
 {
 #undef P_N
 #define P_N "build_mainWin: "
